%% check if hand size is correct;
%**ATTENTION**: the palm should be higher or on the same level
%as the elbow!!!
%I - input depth image
%mask - initial hand mask (1 - for hand location, 0 - for background)
%options:
%
%calibparam - camera calibration parameters
%maxSize - maximal hand size
%verbose - should  information be shown

function newmask = handSizeCheck(I,mask,options)

    verbose = options.verbose;
    calibparam = options.calibparam;
    maxHandSize = options.maxSize;

    %to 3D
    [X,Y]=meshgrid(1:size(I,2),1:size(I,1));
    
    X = (X-calibparam.cx).*I/calibparam.fx;
    Y = (Y-calibparam.cy).*I/calibparam.fy;
    
    linidx = find(mask==1);
   
    %compute main axis
    pc = [X(linidx) Y(linidx) I(linidx)];
    
    %3D hand size check
    np = find(pc(:,3) == min(pc(:,3)));
    np = np(1);
    
    dist = pc - repmat(pc(np,:),size(pc,1),1);
    dist = sqrt(sum(dist.^2,2));
    pc = pc(dist<maxHandSize,:);
    linidx(dist>=maxHandSize)=[];
    
    %prepare for svg
    
    mpc = mean(pc,1);
    pc1 = (pc - repmat(mpc,size(pc,1),1));
    [U,S,~]=svd(pc1','econ');
    
    %make U rotated upwards:
    if (U(2,1)>0)
        U(:,1) = -U(:,1);
    end
    
    proj = pc1*U;
    sizes3D = max(proj,[],1) - min(proj,[],1);
    
    if (verbose)
        disp(sizes3D);
        
        figure(2)
        
        cla(gca)
        plot3(pc(:,1),pc(:,2),pc(:,3),'+');
        
        hold on
        
        as = 100;
        color  = 'rgb';
        
        for i=1:3
            plot3([mpc(1) mpc(1)+as*U(1,i)],[mpc(2) mpc(2)+as*U(2,i)],[mpc(3) mpc(3)+as*U(3,i)],['-' color(i)],'LineWidth',3)
        end
        
        axis equal
        
    end
    
    %check if hand only is segmented
    
    if (sizes3D(1) > maxHandSize)
        
        np = find(proj(:,1) == max(proj(:,1))); %hand always directed somehow up

        gp = abs(proj(:,1) - proj(np(1),1)) >= maxHandSize;
        
        linidx(gp) = [];
        
        if (verbose)
            %control
            disp(['Projection: ' num2str(abs(proj(np(1),1)))]);
            sizes3D = max(proj(~gp,:),[],1) - min(proj(~gp,:),[],1);
            disp(['Control: ' num2str(sizes3D)]);
        end
        
    end
    
    newmask = zeros(size(mask));
    newmask(linidx) = 1;

end